;;
;; Runtime Library for Pathetic Writer
;; Copyright 1996-1998 Ulric Eriksson
;;

; Load the runtime library from SIOD
(require (string-append datadir "/siod/siod.scm"))

; These are missing
(define (caaaar l) (caar (caar l)))
(define (cdaaar l) (cdar (caar l)))
(define (cadaar l) (cadr (caar l)))
(define (cddaar l) (cddr (caar l)))
(define (caadar l) (caar (cdar l)))
(define (cdadar l) (cdar (cdar l)))
(define (caddar l) (cadr (cdar l)))
(define (cdddar l) (cddr (cdar l)))
(define (caaadr l) (caar (cadr l)))
(define (cdaadr l) (cdar (cadr l)))
(define (cadadr l) (cadr (cadr l)))
(define (cddadr l) (cddr (cadr l)))
(define (caaddr l) (caar (cddr l)))
(define (cdaddr l) (cdar (cddr l)))
(define (cadddr l) (cadr (cddr l)))
(define (cddddr l) (cddr (cddr l)))

; Set up the menu
(define (create-menus)
  (load (string-append datadir "/pw/menu.scm")))

(define (init-windows)
  (require (string-append datadir "/plugins/plugin.scm")))

; X string definitions
(require (string-append datadir "/xcommon/StringDefs.scm"))

; Form interface
(require (string-append datadir "/xcommon/form.scm"))


(define (llpr x)
  (puts x))

;; Handle the "position"
(define (make-position row col)
  (list row col))

(define (position-row p)
  (car p))

(define (position-col p)
  (cadr p))

; Find a position not out of bounds
(define (safe-position row col)
  (if (< row 1) (set! row 1))
  (if (> row (max-lines)) (set! row (max-lines)))
  (if (< col 0) (set! col 0))
  (if (> col (col-last-used row)) (set! col (col-last-used row)))
  (make-position row col))

(define (get-point-row)
  (position-row (get-point)))

(define (get-point-col)
  (position-col (get-point)))

(define (set-point-row row)
  (set-point (safe-position row (get-point-col))))

(define (set-point-col col)
  (set-point (safe-position (get-point-row) col)))



(define (<= x y)
  (or (< x y) (= x y)))

(define (>= x y)
  (or (> x y) (= x y)))

(define (at-beginning-of-line p)
  (<= (position-col p) 0))

(define (at-end-of-line p)
  (>= (position-col p) (col-last-used (position-row p))))


(define (find-beginning-of-line p)
  (make-position (position-row p) 0))

(define (find-end-of-line p)
  (make-position (position-row p) (col-last-used (position-row p))))

(define (at-top-of-buffer p)
  (<= (position-row p) 1))

(define (at-bottom-of-buffer p)
  (>= (position-row p) (line-last-used)))

(define (at-beginning-of-buffer p)
  (and (at-top-of-buffer p) (at-beginning-of-line p)))

(define (at-end-of-buffer p)
  (and (at-bottom-of-buffer p) (at-end-of-line p)))


(define (find-beginning-of-buffer)
  (make-position 1 0))

(define (find-end-of-buffer)
  (let ((row (line-last-used)))
    (make-position row (col-last-used row))))


(define (line-forward p)
  (if (< (position-row p) (max-lines))
    (safe-position (+ (position-row p) 1) (position-col p))
    p))

(define (line-backward p)
  (if (at-top-of-buffer p)
    p
    (safe-position (- (position-row p) 1) (position-col p))))


(define (char-forward p)
  (if (at-end-of-line p)
    (find-beginning-of-line (line-forward p))
    (make-position (position-row p) (+ (position-col p) 1))))


(define (char-backward p)
  (if (at-beginning-of-line p)
    (if (at-beginning-of-buffer p)
      p
      (find-end-of-line (line-backward p)))
    (make-position (position-row p) (- (position-col p) 1))))



(define (backward-char)
  (set-point (char-backward (get-point))))

(define (forward-char)
  (set-point (char-forward (get-point))))


(define (set-mark-command)
  (begin
    (set-mark (get-point))
    (llpr "Mark set")))

(define (exchange-point-and-mark)
  (let ((p (get-point)))
    (set-point (get-mark))
    (set-mark p)))

(define (beginning-of-buffer)
  (set-mark (get-point))
  (set-point (find-beginning-of-buffer)))

(define (end-of-buffer)
  (set-mark (get-point))
  (set-point (find-end-of-buffer)))

(define (top-of-buffer)
  (set-point-row 1))

(define (bottom-of-buffer)
  (set-point-row (line-last-used)))

(define (beginning-of-line)
  (set-point-col 0))

(define (end-of-line)
  (set-point-col (col-last-used (get-point-row))))

(define (next-line)
  (set-point (line-forward (get-point))))

(define (previous-line)
  (set-point (line-backward (get-point))))

(define (select-all)
  (let ((p (get-point)))
    (set-mark (find-beginning-of-buffer))
    (set-point (find-end-of-buffer))
    (set-block)
    (set-point p)))

(define (execute-extended-command)
  (execute-interpreter-command 'SIOD))

(define (new-pw)
  (spawn "pw"))

(define (do-help helpfile)
  (spawn (string-append siaghelp " file:" docdir "/" helpfile)))

(define (do-link url)
  (spawn (string-append siaghelp " " url)))

(define (help-contents)
  (do-help "pw/pw.html"))

(define (help-copyright)
  (do-help "pw/COPYING"))

(define (help-for-help)
  (do-help "common/siaghelp.html"))

(define (help-search)
  (do-help "common/search.html"))

(define (keyboard-quit)
  (llpr "Quit"))

(define (no-op)
  (llpr "This command does nothing"))


(define (newline)
  (begin
    (split-line (position-row (get-point))
		(position-col (get-point)))
    (forward-char)
    (change-style (style-follower (get-style (position-row (get-point)))))
    (set-point (get-point))))	; to set format

(define (recenter)
  (set-pr-scr))

(define (delete-char)
  (if (at-end-of-line (get-point))
    (join-lines (position-row (get-point)))
    (remove-text 1)))


(define (delete-char-backward)
  (if (at-beginning-of-buffer (get-point))
    (bell)
    (begin
      (backward-char)
      (delete-char))))

(define (kill-line)
  (let ((row (position-row (get-point)))
	(col (position-col (get-point))))
    (if (at-end-of-line (get-point))
      (join-lines row)
      (begin
	(split-line row col)
	(delete-lines (+ row 1) 1)))
    (set-pr-scr)))

(define (delete-block)
  (let ((ur (position-row (get-blku)))
	(uc (position-col (get-blku)))
	(lr (position-row (get-blkl)))
	(lc (position-col (get-blkl))))
    (split-line lr lc)
    (split-line ur uc)
    (delete-lines (+ ur 1) (+ (- lr ur) 1))
    (join-lines ur)
    (set-pr-scr)))


(define (search-forward)
  (require (string-append datadir "/pw/find.scm"))
  (search-forward))

(define (search-backward)
  (require (string-append datadir "/pw/find.scm"))
  (search-backward))


(define STY_DEFAULT 0)
(define STY_HEADER1 1)
(define STY_HEADER2 2)
(define STY_HEADER3 3)
(define STY_HEADER4 4)
(define STY_HEADER5 5)
(define STY_HEADER6 6)
(define STY_ADDRESS 7)
(define STY_TITLE 8)
(define STY_ABSTRACT 9)
(define STY_PREFORMAT 10)
(define STY_USER1 11)
(define STY_USER2 12)
(define STY_USER3 13)
(define STY_USER4 14)
(define STY_USER5 15)
(define STY_EMBED STY_USER5)


(define (change-style style)
  (set-style (position-row (get-point)) style))

(define (change-hadjust hadj)
  (set-hadjust (position-row (get-point)) hadj))

;(define (change-vadjust vadj)
;  (if (= vadj (bit-and (get-format nil (get-point)) VADJ_MASK))
;    (change-font VADJ_CENTER VADJ_MASK)
;    (change-font vadj VADJ_MASK)))

(define HADJ_MASK (bit-or 4096 8192))
(define HADJ_LEFT 0)
(define HADJ_CENTER 8192)
(define HADJ_RIGHT (bit-or 4096 8192))
(define VADJ_MASK (bit-or 16384 32768))
(define VADJ_TOP 16384)
(define VADJ_CENTER 0)
(define VADJ_BOTTOM (+ 16384 32768))

(define (change-format buf r c name value)
  (let* ((fmt (decode-format (get-format buf r c)))
	 (field (assoc name fmt)))
    (set-cdr! field value)
    (set-format buf r c (encode-format fmt))))

(define (line-format buf r c1 c2 name value)
  (while (< c1 c2)
    (change-format buf r c1 name value)
    (set! c1 (+ c1 1))))

(define (area-format buf r1 c1 r2 c2 name value)
  (let ((r r1))
    (while (<= r r2)
      (line-format buf
		   r1
		   (if (= r r1) c1 0)
		   (if (= r r2) c2 (col-last-used r))
		   name
		   value)
      (set! r (+ r 1))))
  (set-pr-scr))

; If point within block, change a single property for every character in
; the block, then change current format.
; If point outside block, *only* change current format.
(define (new-format name value)
  (let ((r (position-row (get-point)))
	(c (position-col (get-point)))
	(r1 (position-row (get-blku)))
	(c1 (position-col (get-blku)))
	(r2 (position-row (get-blkl)))
	(c2 (position-col (get-blkl))))
    (if (and (or (> r r1)
	         (and (= r r1)
		      (>= c c1)))
             (or (< r r2)
	         (and (= r r2)
		      (<= c c2))))
      (area-format nil r1 c1 r2 c2 name value))
    (let* ((fmt (decode-format (get-format nil r c)))
	   (field (assoc name fmt)))
      (set-cdr! field value)
      (set-current-fmt (encode-format fmt))))
  (set-pr-scr))

(define (toggle-format name)
  (let* ((r (get-point-row))
	 (c (get-point-col))
	 (fmt (decode-format (get-format nil r c)))
	 (field (assoc name fmt))
	 (value (cdr field)))
    (new-format name (if (= value 0) 1 0))))

(define (toggle-vadj newv)
  (let* ((r (get-point-row))
	 (c (get-point-col))
	 (fmt (decode-format (get-format nil r c)))
	 (field (assoc "vadj" fmt))
	 (oldv (cdr field))
	 (value (if (= oldv newv) VADJ_CENTER newv)))
    (new-format "vadj" value)))

(define (exec-siod)
  (execute-interpreter-command 'SIOD))

(define (exec-guile)
  (execute-interpreter-command 'Guile))

(define (exec-python)
  (execute-interpreter-command 'Python))

(define (exec-tcl)
  (execute-interpreter-command 'Tcl))

(define *tooltip-mode* 2)

(define viewer-command "gv")
(define lpr-command "lpr")

(define (preview)
  (let ((fn (string-append "/tmp/siagprint" (number->string (getpid)) ".ps")))
    (if (savematrix fn nil (psformat))
      (spawn (string-append viewer-command " " fn))
      (llpr "Can't make Postscript"))))

(define (print)
  (let ((fn (string-append "/tmp/siagprint" (number->string (getpid)) ".ps")))
    (if (savematrix fn nil (psformat))
      (spawn (string-append lpr-command " " fn))
      (llpr "Can't make Postscript"))))

(define (print-format)
  (let* ((props (form-record "Previewer" "Print Command"))
	 (viewcmd (cdr (assoc "Previewer" props)))
	 (printcmd (cdr (assoc "Print Command" props))))
    (if (not (equal? viewcmd ""))
      (set! viewer-command viewcmd))
    (if (not (equal? printcmd ""))
      (set! lpr-command printcmd))))

; load the fonts
(require (string-append datadir "/common/fonts.scm"))

; and the styles
(define (register-style . ap)
  (register-style-wrapper ap))

(require (string-append datadir "/pw/styles.scm"))


(define (special-char c)
  (let ((text "x"))
    (aset text 0 c)
    (insert-text text)
    (forward-char)))

